#include <exception>
#include <stdexcept>
#include <cstring>

/// \file PhotomatixException.h
/// \brief Contains the definitions for the exceptions thrown by classes in the PhotomatixCL library.
/// 

namespace photomatix
{

	/// <summary>
	/// Indicates an error occurred during library operations
	/// </summary>
#ifdef __swift__
	class PhotomatixException
	{
	public:
		PhotomatixException();
		PhotomatixException(const char *msg);
		~PhotomatixException();
		PhotomatixException(const PhotomatixException &other);
		PhotomatixException& operator=(const PhotomatixException &other);
		const char* what() const noexcept;
	private:
		char* str;
	};
#else
	class PhotomatixException : public std::runtime_error
	{
	public:
		PhotomatixException(const char* msg) : std::runtime_error(msg) { }
	};
#endif

	/// <summary>
	/// Thrown when an image cannot be loaded, or is invalid in some way
	/// </summary>
	class InvalidImageException : public PhotomatixException
	{
	public:
		InvalidImageException(const char* msg) : PhotomatixException(msg) { }
	};

	/// <summary>
	/// Thrown when memory is exhausted while processing
	/// </summary>
#ifdef __swift__
	class OutOfMemoryException : public PhotomatixException
	{
	public:
		OutOfMemoryException();
		OutOfMemoryException(const char *msg) : PhotomatixException(msg) { }
	};
#else
	class OutOfMemoryException : public std::bad_alloc {
	public:
		OutOfMemoryException(const char* msg);
		~OutOfMemoryException();
		OutOfMemoryException(const OutOfMemoryException& other);
		OutOfMemoryException& operator=(const OutOfMemoryException& other);
		const char* what() const noexcept;

	private:
		char* str;
	};
#endif
}
