//
//  ViewController.m
//  PhotomatixLibTestOC
//
//  Created by Matthew Donaldson on 10/12/2024.
//

#import "ViewController.h"

@interface ViewController ()

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<UIImagePickerControllerInfoKey,id> *)info {
    // Get the selected image's URL
    NSURL *imageURL = [info objectForKey:UIImagePickerControllerImageURL];
    
    if (imageURL) {
        NSString *imagePath = [imageURL path];
        NSLog(@"Selected Image Path: %@", imagePath);
        [_txtFileName setText: imagePath];
    }
    
    [picker dismissViewControllerAnimated:YES completion:nil];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [picker dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)btnProcessClicked:(id)sender {
    try {
        // Create the PhotomatixEngine which will store the source files and merge them
        photomatix::PhotomatixEngine engine;
        NSString *imagePath = [_txtFileName text];
        // Add image to the engine
        engine.addImageFromFile(imagePath.UTF8String);
        
        // Presets determine how the images will be merged and processed, and what settings will be used.  They are stored
        // as xmp files.  There are a number included with the project, and you can also create new ones using Photomatix Pro
        NSString *filePath = [[NSBundle mainBundle] pathForResource:@"Detailed"
                                                             ofType:@"xmp"
                                                        inDirectory:@"BuiltinPresets"];
        
        if (filePath) {
            NSLog(@"Preset file path: %@", filePath);
        } else {
            NSLog(@"Preset file not found.");
        }
        
        // Create an HDRPreset object from the xmp file.
        photomatix::HDRPreset preset = photomatix::HDRPreset::createFromFile(filePath.UTF8String);
        
        // Merge (if there are multiple images) or enhance the images with the given preset, returning a result Image object.
        photomatix::Image img = engine.processWithPreset(preset);
        NSLog(@"Got image with width %d x height %d", img.getWidth(), img.getHeight());
        
        // This is a helper method to turn a photomatix::Image object into a UIImage.
        UIImage *uiImg = [PMImageUtils uiImageFromImage:&img];
        _imgResult.image = uiImg;
    }
    catch (photomatix::InvalidImageException &exc) {
        NSLog(@"Got invalid image exception: %s", exc.what());
    }
    catch (photomatix::PhotomatixException &exc) {
        NSLog(@"Got PhotomatixException: %s", exc.what());
    }
    catch (std::exception &exc) {
        NSLog(@"Got std::exception: %s", exc.what());
    }
    catch (...) {
        NSLog(@"Got unknown exception");
    }
}

- (IBAction)btnLoadClicked:(id)sender {
    UIImagePickerController *imagePicker = [[UIImagePickerController alloc] init];
    imagePicker.delegate = self;
    imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    imagePicker.allowsEditing = NO;
    [self presentViewController:imagePicker animated:YES completion:nil];
}
@end
